@@ -167,6 +167,7 @@ typedef struct {
   int isdiskchanged;
   int isbufferfull;
   int speed1x;
+  int isaudio;
   u8 transfileinfo[12];
   u8 lastbuffer;
 
@@ -209,8 +210,8 @@ typedef struct {
   mpegstm_struct mpegstm[2];
 
   int _command;
-  u32 _periodiccycles;
-  u32 _periodictiming;
+  u32 _periodiccycles;  // microseconds * 3
+  u32 _periodictiming;  // microseconds * 3
   u32 _commandtiming;
   CDInterface * cdi;
 
@@ -249,7 +250,11 @@ void FASTCALL 	Cs2WriteByte(u32, u8);
 void FASTCALL 	Cs2WriteWord(u32, u16);
 void FASTCALL 	Cs2WriteLong(u32, u32);
 
+void FASTCALL   Cs2RapidCopyT1(void *dest, u32 count);
+void FASTCALL   Cs2RapidCopyT2(void *dest, u32 count);
+
 void Cs2Exec(u32);
+int Cs2GetTimeToNextSector(void);
 void Cs2Execute(void);
 void Cs2Reset(void);
 void Cs2SetTiming(int);
@@ -260,7 +265,7 @@ void Cs2SetCommandTiming(u8 cmd);
 void Cs2GetStatus(void);                   // 0x00
 void Cs2GetHardwareInfo(void);             // 0x01
 void Cs2GetToc(void);                      // 0x02
-void Cs2GetSessionInfo();                  // 0x03
+void Cs2GetSessionInfo(void);              // 0x03
 void Cs2InitializeCDSystem(void);          // 0x04
 // Open Tray                               // 0x05
 void Cs2EndDataTransfer(void);             // 0x06
