@@ -35,13 +35,12 @@
 extern GKeyFile * keyfile;
 extern yabauseinit_struct yinit;
 
-extern VideoInterface_struct * VIDCoreList[];
-extern SoundInterface_struct * SNDCoreList[];
+extern void YuiSaveState(void);
+extern void YuiLoadState(void);
 
 GtkWidget* create_dialog1(void);
 GtkWidget* create_menu(YuiWindow *);
 
-void yui_run(void);
 void yui_conf(void);
 void yui_resize(guint, guint, gboolean);
 
