@@ -317,7 +317,7 @@ static void yui_sh_init (YuiSh * sh2) {
 }
 
 
-GtkWidget * yui_sh_new(YuiWindow * y, gboolean bMaster) {
+static GtkWidget * yui_sh_new(YuiWindow * y, gboolean bMaster) {
   GtkWidget * dialog;
   GClosure *closureF7; //, *closureF8;
   GtkAccelGroup *accelGroup;
@@ -326,12 +326,8 @@ GtkWidget * yui_sh_new(YuiWindow * y, gboolean bMaster) {
   yui = y;
 
   if (!( yui->state & YUI_IS_INIT )) {
-    /* FIXME:  The first parameter seems to be ignored in these functions,
-     * so NULL is safe; it might be better to consider moving the "yui"
-     * argument first and calling g_signal_connect_swapped() to eliminate
-     * the unknown parameter.  --AC */
-    yui_window_run(NULL, yui);
-    yui_window_pause(NULL, yui);
+    yui_window_run(yui);
+    yui_window_pause(yui);
   }
 
   if ( bMaster && yui_msh ) return GTK_WIDGET(yui_msh);
@@ -464,7 +460,7 @@ static void sh2setRegister( YuiSh *sh2, int nReg, u32 value ) {
 }
 
 void SH2UpdateBreakpointList(YuiSh * sh2) {
-  codebreakpoint_struct *cbp;
+  const codebreakpoint_struct *cbp;
   int i;
 
   gtk_list_store_clear(GTK_LIST_STORE( sh2->bpListStore ));
@@ -474,10 +470,10 @@ void SH2UpdateBreakpointList(YuiSh * sh2) {
   for (i = 0; i < MAX_BREAKPOINTS-1; i++) {
     
     if (cbp[i].addr != 0xFFFFFFFF) {
+      gchar tempstr[20];
       GtkTreeIter iter;
       gtk_list_store_append( GTK_LIST_STORE( sh2->bpListStore ), &iter );
       
-      gchar tempstr[20];
       sprintf(tempstr, "%08X", (int)cbp[i].addr);
       gtk_list_store_set( GTK_LIST_STORE( sh2->bpListStore ), &iter, 0, tempstr, -1 );
     }
@@ -485,7 +481,7 @@ void SH2UpdateBreakpointList(YuiSh * sh2) {
 }
 
 void SH2UpdateMemoryBreakpointList(YuiSh * sh2) {
-  memorybreakpoint_struct *cmbp;
+  const memorybreakpoint_struct *cmbp;
   int i;
 
   gtk_list_store_clear( sh2->mbpListStore );
@@ -564,7 +560,7 @@ static void SH2UpdateCodeList( YuiSh *sh2, u32 addr) {
 static void yui_sh_step( GtkWidget* widget, YuiSh * sh2 ) {
 
   SH2Step(sh2->debugsh);
-  yui_window_invalidate( widget, yui ); /* update all dialogs, including us */
+  yui_window_invalidate( yui ); /* update all dialogs, including us */
 }
 
 static void yui_sh_editedReg( GtkCellRendererText *cellrenderertext,
@@ -587,7 +583,7 @@ static void yui_sh_editedReg( GtkCellRendererText *cellrenderertext,
     sh2setRegister( sh2, i, addr );
     gtk_list_store_set( GTK_LIST_STORE( sh2->regListStore ), &iter, 1, bptext, -1 );
   }
-  yui_window_invalidate( NULL, yui );
+  yui_window_invalidate( yui );
 }
 
 static void yui_sh_editedBp( GtkCellRendererText *cellrenderertext,
@@ -690,6 +686,7 @@ static void yui_sh_editedMbpFlags( GtkCellRendererText *cellrenderertext,
   unsigned int addr;
   memorybreakpoint_struct *cmbp;
   gchar * addr_s;
+  u32 flags = 0;
 
   cmbp = SH2GetMemoryBreakpointList(sh2->debugsh);
 
@@ -703,7 +700,6 @@ static void yui_sh_editedMbpFlags( GtkCellRendererText *cellrenderertext,
 
   endptr = arg2;
     
-  u32 flags = 0;
   while ( *endptr ) {
       
     switch (*endptr) {
@@ -723,7 +719,7 @@ static void yui_sh_editedMbpFlags( GtkCellRendererText *cellrenderertext,
   SH2UpdateMemoryBreakpointList(sh2);
 }
 
-static void debugPauseLoop() { /* secondary gtk event loop for the "breakpoint pause" state */
+static void debugPauseLoop(void) { /* secondary gtk event loop for the "breakpoint pause" state */
 
   while ( !(yui->state & YUI_IS_RUNNING) )
     if ( gtk_main_iteration() ) return;
@@ -731,7 +727,7 @@ static void debugPauseLoop() { /* secondary gtk event loop for the "breakpoint p
 
 static void SH2BreakpointHandler (SH2_struct *context, u32 addr) {
 
-  yui_window_pause(NULL, yui);
+  yui_window_pause(yui);
   {
     sh2regs_struct sh2regs;
     YuiSh* sh2 = YUI_SH(yui_sh_new( yui, context == MSH2 ));
