@@ -40,6 +40,7 @@
 #include "vdp1.h"
 #include "vdp2.h"
 #include "yabause.h"
+#include "yui.h"
 #include "movie.h"
 
 #ifdef HAVE_LIBGL
@@ -68,6 +69,11 @@ u8 *LowWram;
 u8 *BiosRom;
 u8 *BupRam;
 
+/* This flag is set to 1 on every write to backup RAM.  Ports can freely
+ * check or clear this flag to determine when backup RAM has been written,
+ * e.g. for implementing autosave of backup RAM. */
+u8 BupRamWritten;
+
 //////////////////////////////////////////////////////////////////////////////
 
 u8 * T1MemoryInit(u32 size)
@@ -142,7 +148,7 @@ void DummyDeInit(UNUSED Dummy * d)
 
 //////////////////////////////////////////////////////////////////////////////
 
-u8 FASTCALL UnhandledMemoryReadByte(USED_IF_DEBUG u32 addr)
+static u8 FASTCALL UnhandledMemoryReadByte(USED_IF_DEBUG u32 addr)
 {
    LOG("Unhandled byte read %08X\n", (unsigned int)addr);
    return 0;
@@ -150,7 +156,7 @@ u8 FASTCALL UnhandledMemoryReadByte(USED_IF_DEBUG u32 addr)
 
 //////////////////////////////////////////////////////////////////////////////
 
-u16 FASTCALL UnhandledMemoryReadWord(USED_IF_DEBUG u32 addr)
+static u16 FASTCALL UnhandledMemoryReadWord(USED_IF_DEBUG u32 addr)
 {
    LOG("Unhandled word read %08X\n", (unsigned int)addr);
    return 0;
@@ -158,7 +164,7 @@ u16 FASTCALL UnhandledMemoryReadWord(USED_IF_DEBUG u32 addr)
 
 //////////////////////////////////////////////////////////////////////////////
 
-u32 FASTCALL UnhandledMemoryReadLong(USED_IF_DEBUG u32 addr)
+static u32 FASTCALL UnhandledMemoryReadLong(USED_IF_DEBUG u32 addr)
 {
    LOG("Unhandled long read %08X\n", (unsigned int)addr);
    return 0;
@@ -166,162 +172,161 @@ u32 FASTCALL UnhandledMemoryReadLong(USED_IF_DEBUG u32 addr)
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL UnhandledMemoryWriteByte(USED_IF_DEBUG u32 addr, UNUSED u8 val)
+static void FASTCALL UnhandledMemoryWriteByte(USED_IF_DEBUG u32 addr, UNUSED u8 val)
 {
    LOG("Unhandled byte write %08X\n", (unsigned int)addr);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL UnhandledMemoryWriteWord(USED_IF_DEBUG u32 addr, UNUSED u16 val)
+static void FASTCALL UnhandledMemoryWriteWord(USED_IF_DEBUG u32 addr, UNUSED u16 val)
 {
    LOG("Unhandled word write %08X\n", (unsigned int)addr);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL UnhandledMemoryWriteLong(USED_IF_DEBUG u32 addr, UNUSED u32 val)
+static void FASTCALL UnhandledMemoryWriteLong(USED_IF_DEBUG u32 addr, UNUSED u32 val)
 {
    LOG("Unhandled long write %08X\n", (unsigned int)addr);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-u8 FASTCALL HighWramMemoryReadByte(u32 addr)
+static u8 FASTCALL HighWramMemoryReadByte(u32 addr)
 {
    return T2ReadByte(HighWram, addr & 0xFFFFF);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-u16 FASTCALL HighWramMemoryReadWord(u32 addr)
+static u16 FASTCALL HighWramMemoryReadWord(u32 addr)
 {
    return T2ReadWord(HighWram, addr & 0xFFFFF);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-u32 FASTCALL HighWramMemoryReadLong(u32 addr)
+static u32 FASTCALL HighWramMemoryReadLong(u32 addr)
 {
    return T2ReadLong(HighWram, addr & 0xFFFFF);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL HighWramMemoryWriteByte(u32 addr, u8 val)
+static void FASTCALL HighWramMemoryWriteByte(u32 addr, u8 val)
 {
    T2WriteByte(HighWram, addr & 0xFFFFF, val);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL HighWramMemoryWriteWord(u32 addr, u16 val)
+static void FASTCALL HighWramMemoryWriteWord(u32 addr, u16 val)
 {
    T2WriteWord(HighWram, addr & 0xFFFFF, val);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL HighWramMemoryWriteLong(u32 addr, u32 val)
+static void FASTCALL HighWramMemoryWriteLong(u32 addr, u32 val)
 {
    T2WriteLong(HighWram, addr & 0xFFFFF, val);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-u8 FASTCALL LowWramMemoryReadByte(u32 addr)
+static u8 FASTCALL LowWramMemoryReadByte(u32 addr)
 {
    return T2ReadByte(LowWram, addr & 0xFFFFF);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-u16 FASTCALL LowWramMemoryReadWord(u32 addr)
+static u16 FASTCALL LowWramMemoryReadWord(u32 addr)
 {
    return T2ReadWord(LowWram, addr & 0xFFFFF);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-u32 FASTCALL LowWramMemoryReadLong(u32 addr)
+static u32 FASTCALL LowWramMemoryReadLong(u32 addr)
 {
    return T2ReadLong(LowWram, addr & 0xFFFFF);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL LowWramMemoryWriteByte(u32 addr, u8 val)
+static void FASTCALL LowWramMemoryWriteByte(u32 addr, u8 val)
 {
    T2WriteByte(LowWram, addr & 0xFFFFF, val);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL LowWramMemoryWriteWord(u32 addr, u16 val)
+static void FASTCALL LowWramMemoryWriteWord(u32 addr, u16 val)
 {
    T2WriteWord(LowWram, addr & 0xFFFFF, val);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL LowWramMemoryWriteLong(u32 addr, u32 val)
+static void FASTCALL LowWramMemoryWriteLong(u32 addr, u32 val)
 {
    T2WriteLong(LowWram, addr & 0xFFFFF, val);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-u8 FASTCALL BiosRomMemoryReadByte(u32 addr)
+static u8 FASTCALL BiosRomMemoryReadByte(u32 addr)
 {
    return T2ReadByte(BiosRom, addr & 0x7FFFF);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-u16 FASTCALL BiosRomMemoryReadWord(u32 addr)
+static u16 FASTCALL BiosRomMemoryReadWord(u32 addr)
 {
    return T2ReadWord(BiosRom, addr & 0x7FFFF);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-u32 FASTCALL BiosRomMemoryReadLong(u32 addr)
+static u32 FASTCALL BiosRomMemoryReadLong(u32 addr)
 {
    return T2ReadLong(BiosRom, addr & 0x7FFFF);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL BiosRomMemoryWriteByte(UNUSED u32 addr, UNUSED u8 val)
+static void FASTCALL BiosRomMemoryWriteByte(UNUSED u32 addr, UNUSED u8 val)
 {
    // read-only
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL BiosRomMemoryWriteWord(UNUSED u32 addr, UNUSED u16 val)
+static void FASTCALL BiosRomMemoryWriteWord(UNUSED u32 addr, UNUSED u16 val)
 {
    // read-only
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-
-void FASTCALL BiosRomMemoryWriteLong(UNUSED u32 addr, UNUSED u32 val)
+static void FASTCALL BiosRomMemoryWriteLong(UNUSED u32 addr, UNUSED u32 val)
 {
    // read-only
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-u8 FASTCALL BupRamMemoryReadByte(u32 addr)
+static u8 FASTCALL BupRamMemoryReadByte(u32 addr)
 {
    return T1ReadByte(BupRam, addr & 0xFFFF);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-u16 FASTCALL BupRamMemoryReadWord(USED_IF_DEBUG u32 addr)
+static u16 FASTCALL BupRamMemoryReadWord(USED_IF_DEBUG u32 addr)
 {
    LOG("bup\t: BackupRam read word - %08X\n", addr);
    return 0;
@@ -329,7 +334,7 @@ u16 FASTCALL BupRamMemoryReadWord(USED_IF_DEBUG u32 addr)
 
 //////////////////////////////////////////////////////////////////////////////
 
-u32 FASTCALL BupRamMemoryReadLong(USED_IF_DEBUG u32 addr)
+static u32 FASTCALL BupRamMemoryReadLong(USED_IF_DEBUG u32 addr)
 {
    LOG("bup\t: BackupRam read long - %08X\n", addr);
    return 0;
@@ -337,31 +342,32 @@ u32 FASTCALL BupRamMemoryReadLong(USED_IF_DEBUG u32 addr)
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL BupRamMemoryWriteByte(u32 addr, u8 val)
+static void FASTCALL BupRamMemoryWriteByte(u32 addr, u8 val)
 {
    T1WriteByte(BupRam, (addr & 0xFFFF) | 0x1, val);
+   BupRamWritten = 1;
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL BupRamMemoryWriteWord(USED_IF_DEBUG u32 addr, UNUSED u16 val)
+static void FASTCALL BupRamMemoryWriteWord(USED_IF_DEBUG u32 addr, UNUSED u16 val)
 {
    LOG("bup\t: BackupRam write word - %08X\n", addr);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL BupRamMemoryWriteLong(USED_IF_DEBUG u32 addr, UNUSED u32 val)
+static void FASTCALL BupRamMemoryWriteLong(USED_IF_DEBUG u32 addr, UNUSED u32 val)
 {
    LOG("bup\t: BackupRam write long - %08X\n", addr);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FillMemoryArea(unsigned short start, unsigned short end,
-                    readbytefunc r8func, readwordfunc r16func,
-                    readlongfunc r32func, writebytefunc w8func,
-                    writewordfunc w16func, writelongfunc w32func)
+static void FillMemoryArea(unsigned short start, unsigned short end,
+                           readbytefunc r8func, readwordfunc r16func,
+                           readlongfunc r32func, writebytefunc w8func,
+                           writewordfunc w16func, writelongfunc w32func)
 {
    int i;
 
@@ -926,7 +932,9 @@ void MappedMemoryLoadExec(const char *filename, u32 pc)
    YabauseSpeedySetup();
 
    MappedMemoryLoad(filename, pc);
+   SH2GetRegisters(MSH2, &MSH2->regs);
    MSH2->regs.PC = pc;
+   SH2SetRegisters(MSH2, &MSH2->regs);
 }
 
 //////////////////////////////////////////////////////////////////////////////
@@ -971,6 +979,19 @@ void FormatBackupRam(void *mem, u32 size)
 
 //////////////////////////////////////////////////////////////////////////////
 
+// FIXME: Here's a (possibly incomplete) list of data that should be added
+// to the next version of the save state file:
+//    yabsys.DecilineStop (new format)
+//    yabsys.SH2CycleFrac (new field)
+//    yabsys.DecilineUSed (new field)
+//    yabsys.UsecFrac (new field)
+//    [scsp2.c] It would be nice to redo the format entirely because so
+//              many fields have changed format/size from the old scsp.c
+//    [scsp2.c] scsp_clock, scsp_clock_frac, ScspState.sample_timer (timing)
+//    [scsp2.c] cdda_buf, cdda_next_in, cdda_next_out (CDDA buffer)
+//    [sh2core.c] frc.div changed to frc.shift
+//    [sh2core.c] wdt probably needs to be written as well
+
 int YabSaveState(const char *filename)
 {
    u32 i;
@@ -982,9 +1003,16 @@ int YabSaveState(const char *filename)
    int outputwidth;
    int outputheight;
    int movieposition;
+   int temp;
+   u32 temp32;
+
+   check.done = 0;
+   check.size = 0;
 
    //use a second set of savestates for movies
-   MakeMovieStateName(filename);
+   filename = MakeMovieStateName(filename);
+   if (!filename)
+      return -1;
 
    if ((fp = fopen(filename, "wb")) == NULL)
       return -1;
@@ -1035,9 +1063,12 @@ int YabSaveState(const char *filename)
    ywrite(&check, (void *)&yabsys.LineCount, sizeof(int), 1, fp);
    ywrite(&check, (void *)&yabsys.VBlankLineCount, sizeof(int), 1, fp);
    ywrite(&check, (void *)&yabsys.MaxLineCount, sizeof(int), 1, fp);
-   ywrite(&check, (void *)&yabsys.DecilineStop, sizeof(int), 1, fp);
-   ywrite(&check, (void *)&yabsys.Duf, sizeof(int), 1, fp);
-   ywrite(&check, (void *)&yabsys.CycleCountII, sizeof(u32), 1, fp);
+   temp = yabsys.DecilineStop >> YABSYS_TIMING_BITS;
+   ywrite(&check, (void *)&temp, sizeof(int), 1, fp);
+   temp = (yabsys.CurSH2FreqType == CLKTYPE_26MHZ) ? 268 : 286;
+   ywrite(&check, (void *)&temp, sizeof(int), 1, fp);
+   temp32 = (yabsys.UsecFrac * temp / 10) >> YABSYS_TIMING_BITS;
+   ywrite(&check, (void *)&temp32, sizeof(u32), 1, fp);
    ywrite(&check, (void *)&yabsys.CurSH2FreqType, sizeof(int), 1, fp);
    ywrite(&check, (void *)&yabsys.IsPal, sizeof(int), 1, fp);
 
@@ -1096,8 +1127,12 @@ int YabLoadState(const char *filename)
    int curroutputwidth;
    int curroutputheight;
    int movieposition;
+   int temp;
+   u32 temp32;
 
-   MakeMovieStateName(filename);
+   filename = MakeMovieStateName(filename);
+   if (!filename)
+      return -1;
 
    if ((fp = fopen(filename, "rb")) == NULL)
       return -1;
@@ -1260,11 +1295,13 @@ int YabLoadState(const char *filename)
    yread(&check, (void *)&yabsys.LineCount, sizeof(int), 1, fp);
    yread(&check, (void *)&yabsys.VBlankLineCount, sizeof(int), 1, fp);
    yread(&check, (void *)&yabsys.MaxLineCount, sizeof(int), 1, fp);
-   yread(&check, (void *)&yabsys.DecilineStop, sizeof(int), 1, fp);
-   yread(&check, (void *)&yabsys.Duf, sizeof(int), 1, fp);
-   yread(&check, (void *)&yabsys.CycleCountII, sizeof(u32), 1, fp);
+   yread(&check, (void *)&temp, sizeof(int), 1, fp);
+   yread(&check, (void *)&temp, sizeof(int), 1, fp);
+   yread(&check, (void *)&temp32, sizeof(u32), 1, fp);
    yread(&check, (void *)&yabsys.CurSH2FreqType, sizeof(int), 1, fp);
    yread(&check, (void *)&yabsys.IsPal, sizeof(int), 1, fp);
+   YabauseChangeTiming(yabsys.CurSH2FreqType);
+   yabsys.UsecFrac = (temp32 << YABSYS_TIMING_BITS) * temp / 10;
 
    if (headerversion > 1) {
 
@@ -1334,7 +1371,7 @@ int YabLoadStateSlot(const char *dirpath, u8 slot)
 
 //////////////////////////////////////////////////////////////////////////////
 
-int MappedMemoryAddMatch(u32 addr, u32 val, int searchtype, result_struct *result, u32 *numresults)
+static int MappedMemoryAddMatch(u32 addr, u32 val, int searchtype, result_struct *result, u32 *numresults)
 {
    result[numresults[0]].addr = addr;
    result[numresults[0]].val = val;
