@@ -50,6 +50,7 @@ extern "C"
 	
 	#include "PerQt.h"
 #ifdef Q_OS_WIN
+	#include <qt_windows.h>
 	#include "../windows/cd.h"
 #endif
 }
@@ -59,12 +60,14 @@ extern "C"
 
 class UIYabause;
 class Settings;
+class VolatileSettings;
 class QWidget;
 
 namespace QtYabause
 {
-	UIYabause* mainWindow();
-	Settings* settings();
+	UIYabause* mainWindow( bool create = true );
+	Settings* settings( bool create = true );
+	VolatileSettings* volatileSettings( bool create = true );
 	int setTranslationFile();
 	int logTranslation();
 	void closeTranslation();
