@@ -30,7 +30,7 @@ class YabauseGL : public QGLWidget
 public:
 	YabauseGL( QWidget* parent = 0 );
 	
-	void updateView( const QSize& size );
+	void updateView( const QSize& size = QSize() );
 
 protected:
 	virtual void showEvent( QShowEvent* event );
