@@ -27,12 +27,14 @@
 #include <QDir>
 #include <QList>
 
+extern "C" {
 extern M68K_struct* M68KCoreList[];
 extern SH2Interface_struct* SH2CoreList[];
 extern PerInterface_struct* PERCoreList[];
 extern CDInterface* CDCoreList[];
 extern SoundInterface_struct* SNDCoreList[];
 extern VideoInterface_struct* VIDCoreList[];
+}
 
 struct Item
 {
@@ -158,6 +160,7 @@ void UISettings::tbBrowse_clicked()
 void UISettings::on_cbInput_currentIndexChanged( int id )
 {
 	PerInterface_struct* core = QtYabause::getPERCore( cbInput->itemData( id ).toInt() );
+        core->Init();
 	
 	Q_ASSERT( core );
 	
@@ -211,6 +214,8 @@ void UISettings::loadSettings()
 	leCdRom->setText( s->value( "General/CdRomISO" ).toString() );
 	leSaveStates->setText( s->value( "General/SaveStates", QApplication::applicationDirPath() ).toString() );
 	leTranslation->setText( s->value( "General/Translation" ).toString() );
+	cbEnableFrameSkipLimiter->setChecked( s->value( "General/EnableFrameSkipLimiter" ).toBool() );
+	cbShowFPS->setChecked( s->value( "General/ShowFPS" ).toBool() );
 
 	// video
 	cbVideoCore->setCurrentIndex( cbVideoCore->findData( s->value( "Video/VideoCore", QtYabause::defaultVIDCore().id ).toInt() ) );
@@ -247,6 +252,8 @@ void UISettings::saveSettings()
 	s->setValue( "General/CdRomISO", leCdRom->text() );
 	s->setValue( "General/SaveStates", leSaveStates->text() );
 	s->setValue( "General/Translation", leTranslation->text() );
+	s->setValue( "General/EnableFrameSkipLimiter", cbEnableFrameSkipLimiter->isChecked() );
+	s->setValue( "General/ShowFPS", cbShowFPS->isChecked() );
 
 	// video
 	s->setValue( "Video/VideoCore", cbVideoCore->itemData( cbVideoCore->currentIndex() ).toInt() );
