@@ -18,6 +18,11 @@
     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
+// Quick hack so nobody else needs to know we're using a different header file
+#ifdef USE_SCSP2
+#include "scsp2.h"  // defines SCSP_H
+#endif
+
 #ifndef SCSP_H
 #define SCSP_H
 
@@ -31,14 +36,14 @@ typedef struct
 {
    int id;
    const char *Name;
-   int (*Init)();
-   void (*DeInit)();
-   int (*Reset)();
+   int (*Init)(void);
+   void (*DeInit)(void);
+   int (*Reset)(void);
    int (*ChangeVideoFormat)(int vertfreq);
    void (*UpdateAudio)(u32 *leftchanbuffer, u32 *rightchanbuffer, u32 num_samples);
-   u32 (*GetAudioSpace)();
-   void (*MuteAudio)();
-   void (*UnMuteAudio)();
+   u32 (*GetAudioSpace)(void);
+   void (*MuteAudio)(void);
+   void (*UnMuteAudio)(void);
    void (*SetVolume)(int volume);
 } SoundInterface_struct;
 
@@ -81,8 +86,10 @@ void FASTCALL SoundRamWriteLong(u32 addr, u32 val);
 
 int ScspInit(int coreid);
 int ScspChangeSoundCore(int coreid);
+void ScspSetFrameAccurate(int on);
 void ScspDeInit(void);
-void M68KReset(void);
+void M68KStart(void);
+void M68KStop(void);
 void ScspReset(void);
 int ScspChangeVideoFormat(int type);
 void M68KExec(s32 cycles);
@@ -95,8 +102,8 @@ void ScspSlotDebugStats(u8 slotnum, char *outstring);
 void ScspCommonControlRegisterDebugStats(char *outstring);
 int ScspSlotDebugSaveRegisters(u8 slotnum, const char *filename);
 int ScspSlotDebugAudioSaveWav(u8 slotnum, const char *filename);
-void ScspMuteAudio();
-void ScspUnMuteAudio();
+void ScspMuteAudio(void);
+void ScspUnMuteAudio(void);
 void ScspSetVolume(int volume);
 
 void FASTCALL scsp_w_b(u32, u8);
@@ -115,17 +122,21 @@ void scsp_midi_out_send(u8 data);
 u8 scsp_midi_in_read(void);
 u8 scsp_midi_out_read(void);
 void scsp_update(s32 *bufL, s32 *bufR, u32 len);
+void scsp_update_ca(void);
 void scsp_update_timer(u32 len);
 
-u32 M68KDisasm(u32 addr, char *outstring);
-void M68KStep();
+u32 FASTCALL c68k_word_read(const u32 adr);
+
+void M68KStep(void);
+void M68KSync(void);
+void M68KWriteNotify(u32 address, u32 size);
 void M68KGetRegisters(m68kregs_struct *regs);
 void M68KSetRegisters(m68kregs_struct *regs);
 void M68KSetBreakpointCallBack(void (*func)(u32));
 int M68KAddCodeBreakpoint(u32 addr);
-void M68KSortCodeBreakpoints();
+void M68KSortCodeBreakpoints(void);
 int M68KDelCodeBreakpoint(u32 addr);
-m68kcodebreakpoint_struct *M68KGetBreakpointList();
-void M68KClearCodeBreakpoints();
+m68kcodebreakpoint_struct *M68KGetBreakpointList(void);
+void M68KClearCodeBreakpoints(void);
 
 #endif
