@@ -18,8 +18,8 @@
     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
-#ifndef SH2_H
-#define SH2_H
+#ifndef SH2CORE_H
+#define SH2CORE_H
 
 #include "core.h"
 #include "memory.h"
@@ -330,7 +330,7 @@ typedef struct
    struct
    {
       u32 leftover;
-      u32 div;
+      u32 shift;
    } frc;
 
    struct
@@ -338,7 +338,7 @@ typedef struct
         int isenable;
         int isinterval;
         u32 leftover;
-        u32 div;
+        u32 shift;
    } wdt;
 
    interrupt_struct interrupts[MAX_INTERRUPTS];
@@ -354,68 +354,53 @@ typedef struct
    u8 breakpointEnabled;
    breakpoint_struct bp;
 
-   /* Data used by PSP SH-2 core */
-   struct
-   {
-      /* Flag indicating whether we should check for interrupts (set by
-       * instructions that modify SR) */
-      int need_interrupt_check;
-
-      /* Currently-executing block */
-      struct JitEntry_ *current_entry;
-
-      /* PC at end of last SH2->Exec() call (used to detect interrupts) */
-      u32 current_PC;
-
-      /* Call stack for optimizing BSR/JSR/RTS */
-      void *call_stack;
-      /* Index of next call stack entry to write */
-      int call_stack_top;
-
-      /* Cached (most-recently-used) page index and associated pointer */
-      u32 cached_page;  // Bits 19-31 of the last used direct-access page
-      u8 *cached_ptr;   // Pointer to direct-access page memory
-
-      /* Pending branch type (to be executed after "delay" cycles) */
-      enum {SH2BRTYPE_NONE = 0, // No branch pending
-            SH2BRTYPE_DYNAMIC,  // Unconditional dynamic branch/jump
-            SH2BRTYPE_STATIC,   // Unconditional static (relative) branch
-            SH2BRTYPE_JSR,      // Subroutine call (either JSR or BSR)
-            SH2BRTYPE_RTS,
-            SH2BRTYPE_BT,
-            SH2BRTYPE_BF,
-            SH2BRTYPE_BT_S,
-            SH2BRTYPE_BF_S,
-           } branch_type;
-      u32 branch_target;    // Target address
-      u32 branch_cond;      // For SH2BRTYPE_BTF, nonzero if branch is taken
-
-      /* Saved values for optimized division (used when tracing) */
-      struct
-      {
-         u32 target_PC;     // PC of final DIV1 instruction (0 if none active)
-         int Rquo, Rrem;    // Register numbers (quotient and remainder)
-         u32 quo, rem, SR;  // Result values for registers
-      } div_data;
-   } psp;
 } SH2_struct;
 
 typedef struct
 {
    int id;
    const char *Name;
-   int (*Init)();
-   void (*DeInit)();
-   int (*Reset)();
+
+   int (*Init)(void);
+   void (*DeInit)(void);
+   void (*Reset)(SH2_struct *context);
    void FASTCALL (*Exec)(SH2_struct *context, u32 cycles);
+
+   void (*GetRegisters)(SH2_struct *context, sh2regs_struct *regs);
+   u32 (*GetGPR)(SH2_struct *context, int num);
+   u32 (*GetSR)(SH2_struct *context);
+   u32 (*GetGBR)(SH2_struct *context);
+   u32 (*GetVBR)(SH2_struct *context);
+   u32 (*GetMACH)(SH2_struct *context);
+   u32 (*GetMACL)(SH2_struct *context);
+   u32 (*GetPR)(SH2_struct *context);
+   u32 (*GetPC)(SH2_struct *context);
+
+   void (*SetRegisters)(SH2_struct *context, const sh2regs_struct *regs);
+   void (*SetGPR)(SH2_struct *context, int num, u32 value);
+   void (*SetSR)(SH2_struct *context, u32 value);
+   void (*SetGBR)(SH2_struct *context, u32 value);
+   void (*SetVBR)(SH2_struct *context, u32 value);
+   void (*SetMACH)(SH2_struct *context, u32 value);
+   void (*SetMACL)(SH2_struct *context, u32 value);
+   void (*SetPR)(SH2_struct *context, u32 value);
+   void (*SetPC)(SH2_struct *context, u32 value);
+
+   void (*SendInterrupt)(SH2_struct *context, u8 vector, u8 level);
+   int (*GetInterrupts)(SH2_struct *context,
+                        interrupt_struct interrupts[MAX_INTERRUPTS]);
+   void (*SetInterrupts)(SH2_struct *context, int num_interrupts,
+                         const interrupt_struct interrupts[MAX_INTERRUPTS]);
+
    void (*WriteNotify)(u32 start, u32 length);
 } SH2Interface_struct;
 
 extern SH2_struct *MSH2;
 extern SH2_struct *SSH2;
+extern SH2Interface_struct *SH2Core;
 
 int SH2Init(int coreid);
-void SH2DeInit();
+void SH2DeInit(void);
 void SH2Reset(SH2_struct *context);
 void SH2PowerOn(SH2_struct *context);
 void FASTCALL SH2Exec(SH2_struct *context, u32 cycles);
@@ -477,4 +462,8 @@ void FASTCALL SSH2InputCaptureWriteWord(u32 addr, u16 data);
 int SH2SaveState(SH2_struct *context, FILE *fp);
 int SH2LoadState(SH2_struct *context, FILE *fp, int version, int size);
 
+#if defined(SH2_DYNAREC)
+extern SH2Interface_struct SH2Dynarec;
+#endif
+
 #endif
