@@ -20,9 +20,8 @@
 
 #ifndef CD_HH
 #define CD_HH
-
-#include <ddk/ntddcdrm.h>
-#include <ddk/ntddscsi.h>
+#include <ntddcdrm.h>
+#include <ntddscsi.h>
 #ifdef HAVE_WNASPI32_H
 #include <wnaspi32.h>
 #endif
@@ -32,18 +31,20 @@
 #define CDCORE_ASPI     3
 
 int SPTICDInit(const char *);
-int SPTICDDeInit();
+void SPTICDDeInit();
 int SPTICDGetStatus();
 s32 SPTICDReadTOC(u32 *);
 int SPTICDReadSectorFAD(u32, void *);
+void SPTICDReadAheadFAD(u32);
 
 extern CDInterface ArchCD;
 
 int ASPICDInit(const char *);
-int ASPICDDeInit();
+void ASPICDDeInit();
 int ASPICDGetStatus();
 s32 ASPICDReadTOC(u32 *);
 int ASPICDReadSectorFAD(u32, void *);
+void ASPICDReadAheadFAD(u32);
 
 extern CDInterface ASPICD;
 
