@@ -31,7 +31,7 @@
 int PERDXInit(void);
 void PERDXDeInit(void);
 int PERDXHandleEvents(void);
-void YuiCaptureVideo(void);
+//void YuiCaptureVideo(void);
 int Check_Skip_Key();
 
 PerInterface_struct PERDIRECTX = {
@@ -42,8 +42,8 @@ PERDXDeInit,
 PERDXHandleEvents
 };
 
-extern HWND YabWin;
-extern HINSTANCE y_hInstance;
+HWND YabWin;
+HINSTANCE y_hInstance;
 
 LPDIRECTINPUT8 lpDI8 = NULL;
 LPDIRECTINPUTDEVICE8 lpDIDevice[256]; // I hope that's enough
@@ -85,8 +85,8 @@ BOOL CALLBACK EnumPeripheralsCallback (LPCDIDEVICEINSTANCE lpddi, LPVOID pvRef)
        GET_DIDEVICE_TYPE(lpddi->dwDevType) == DI8DEVTYPE_JOYSTICK ||
        GET_DIDEVICE_TYPE(lpddi->dwDevType) == DI8DEVTYPE_KEYBOARD)
    {     
-      if (IDirectInput8_CreateDevice(lpDI8, &lpddi->guidInstance, &lpDIDevice[numdevices],
-          NULL) == DI_OK)
+      if (SUCCEEDED(IDirectInput8_CreateDevice(lpDI8, &lpddi->guidInstance, &lpDIDevice[numdevices],
+          NULL) ))
          numdevices++;
    }
 
@@ -128,8 +128,8 @@ int PERDXInit(void)
    memset(pad, 0, sizeof(pad));
    memset(paddevice, 0, sizeof(paddevice));
 
-   if ((ret = DirectInput8Create(GetModuleHandle(NULL), DIRECTINPUT_VERSION,
-       &IID_IDirectInput8, (LPVOID *)&lpDI8, NULL)) != DI_OK)
+   if (FAILED((ret = DirectInput8Create(GetModuleHandle(NULL), DIRECTINPUT_VERSION,
+       &IID_IDirectInput8, (LPVOID *)&lpDI8, NULL)) ))
    {
       sprintf(tempstr, "DirectInput8Create error: %s - %s", DXGetErrorString8(ret), DXGetErrorDescription8(ret));
       MessageBox (NULL, _16(tempstr), _16("Error"),  MB_OK | MB_ICONINFORMATION);
@@ -139,23 +139,23 @@ int PERDXInit(void)
    IDirectInput8_EnumDevices(lpDI8, DI8DEVCLASS_ALL, EnumPeripheralsCallback,
                       NULL, DIEDFL_ATTACHEDONLY);
 
-   if ((ret = IDirectInput8_CreateDevice(lpDI8, &GUID_SysKeyboard, &lpDIDevice[0],
-       NULL)) != DI_OK)
+   if (FAILED((ret = IDirectInput8_CreateDevice(lpDI8, &GUID_SysKeyboard, &lpDIDevice[0],
+       NULL)) ))
    {
       sprintf(tempstr, "IDirectInput8_CreateDevice error: %s - %s", DXGetErrorString8(ret), DXGetErrorDescription8(ret));
       MessageBox (NULL, _16(tempstr), _16("Error"),  MB_OK | MB_ICONINFORMATION);
       return -1;
    }
 
-   if ((ret = IDirectInputDevice8_SetDataFormat(lpDIDevice[0], &c_dfDIKeyboard)) != DI_OK)
+   if (FAILED((ret = IDirectInputDevice8_SetDataFormat(lpDIDevice[0], &c_dfDIKeyboard)) ))
    {
       sprintf(tempstr, "IDirectInputDevice8_SetDataFormat error: %s - %s", DXGetErrorString8(ret), DXGetErrorDescription8(ret));
       MessageBox (NULL, _16(tempstr), _16("Error"),  MB_OK | MB_ICONINFORMATION);
       return -1;
    }
 
-   if ((ret = IDirectInputDevice8_SetCooperativeLevel(lpDIDevice[0], YabWin,
-       DISCL_FOREGROUND | DISCL_NONEXCLUSIVE | DISCL_NOWINKEY)) != DI_OK)
+   if (FAILED((ret = IDirectInputDevice8_SetCooperativeLevel(lpDIDevice[0], YabWin,
+       DISCL_FOREGROUND | DISCL_NONEXCLUSIVE | DISCL_NOWINKEY)) ))
    {
       sprintf(tempstr, "IDirectInputDevice8_SetCooperativeLevel error: %s - %s", DXGetErrorString8(ret), DXGetErrorDescription8(ret));
       MessageBox (NULL, _16(tempstr), _16("Error"),  MB_OK | MB_ICONINFORMATION);
@@ -169,7 +169,7 @@ int PERDXInit(void)
    dipdw.dwData = 8; // should be enough
 
    // Setup Buffered input
-   if ((ret = IDirectInputDevice8_SetProperty(lpDIDevice[0], DIPROP_BUFFERSIZE, &dipdw.diph)) != DI_OK)
+   if (FAILED((ret = IDirectInputDevice8_SetProperty(lpDIDevice[0], DIPROP_BUFFERSIZE, &dipdw.diph)) ))
    {
       sprintf(tempstr, "IDirectInputDevice8_SetProperty error: %s - %s", DXGetErrorString8(ret), DXGetErrorDescription8(ret));
       MessageBox (NULL, _16(tempstr), _16("Error"),  MB_OK | MB_ICONINFORMATION);
@@ -321,8 +321,8 @@ void PERDXLoadDevices(char *inifilename)
             StringToGUID(tempstr, &guid);
 
             // Ok, now that we've got the GUID of the device, let's set it up
-            if (IDirectInput8_CreateDevice(lpDI8, &guid, &lpDIDevice[padindex],
-               NULL) != DI_OK)
+            if (FAILED(IDirectInput8_CreateDevice(lpDI8, &guid, &lpDIDevice[padindex],
+               NULL) ))
             {
                curdevice->lpDIDevice = NULL;
                curdevice->emulatetype = 0;
@@ -333,12 +333,12 @@ void PERDXLoadDevices(char *inifilename)
 
             didc.dwSize = sizeof(DIDEVCAPS);
 
-            if (IDirectInputDevice8_GetCapabilities(lpDIDevice[padindex], &didc) != DI_OK)
+            if (FAILED(IDirectInputDevice8_GetCapabilities(lpDIDevice[padindex], &didc) ))
                continue;
 
             if (GET_DIDEVICE_TYPE(didc.dwDevType) == DI8DEVTYPE_KEYBOARD)
             {
-               if (IDirectInputDevice8_SetDataFormat(lpDIDevice[padindex], &c_dfDIKeyboard) != DI_OK)
+               if (FAILED(IDirectInputDevice8_SetDataFormat(lpDIDevice[padindex], &c_dfDIKeyboard) ))
                   continue;
                curdevice->type = TYPE_KEYBOARD;
                coopflags |= DISCL_NOWINKEY;
@@ -346,13 +346,13 @@ void PERDXLoadDevices(char *inifilename)
             else if (GET_DIDEVICE_TYPE(didc.dwDevType) == DI8DEVTYPE_GAMEPAD ||
                GET_DIDEVICE_TYPE(didc.dwDevType) == DI8DEVTYPE_JOYSTICK)
             {
-               if (IDirectInputDevice8_SetDataFormat(lpDIDevice[padindex], &c_dfDIJoystick2) != DI_OK)
+               if (FAILED(IDirectInputDevice8_SetDataFormat(lpDIDevice[padindex], &c_dfDIJoystick2) ))
                   continue;
                curdevice->type = TYPE_JOYSTICK;
             }
             else if (GET_DIDEVICE_TYPE(didc.dwDevType) == DI8DEVTYPE_MOUSE)
             {
-               if (IDirectInputDevice8_SetDataFormat(lpDIDevice[padindex], &c_dfDIMouse2) != DI_OK)
+               if (FAILED(IDirectInputDevice8_SetDataFormat(lpDIDevice[padindex], &c_dfDIMouse2) ))
                   continue;
                curdevice->type = TYPE_MOUSE;
                coopflags = DISCL_FOREGROUND | DISCL_EXCLUSIVE;
@@ -369,7 +369,7 @@ void PERDXLoadDevices(char *inifilename)
             dipdw.dwData = 8; // should be enough
 
             // Setup Buffered input
-            if (IDirectInputDevice8_SetProperty(lpDIDevice[padindex], DIPROP_BUFFERSIZE, &dipdw.diph) != DI_OK)
+            if (FAILED(IDirectInputDevice8_SetProperty(lpDIDevice[padindex], DIPROP_BUFFERSIZE, &dipdw.diph)))
                continue;
 
             IDirectInputDevice8_Acquire(lpDIDevice[padindex]);
@@ -472,8 +472,8 @@ void PollKeys(void)
       size = 8;
 
       // Poll events
-      if (IDirectInputDevice8_GetDeviceData(paddevice[i].lpDIDevice,
-          sizeof(DIDEVICEOBJECTDATA), didod, &size, 0) != DI_OK)
+      if (FAILED(IDirectInputDevice8_GetDeviceData(paddevice[i].lpDIDevice,
+          sizeof(DIDEVICEOBJECTDATA), didod, &size, 0)))
       {
          if (hr == DIERR_INPUTLOST || hr == DIERR_NOTACQUIRED)
          {
@@ -640,22 +640,14 @@ void PollKeys(void)
 
 //////////////////////////////////////////////////////////////////////////////
 
-int FrameAdvanceKeyDown=0;
-static DWORD tgtime;
-
 int PERDXHandleEvents(void)
 {
    PollKeys();
 
-   	if (Check_Skip_Key())
-	{
-		FrameAdvanceVariable = NeedAdvance;
-		tgtime = timeGetTime();
-	}
-
    if (YabauseExec() != 0)
       return -1;
 
+   Update_RAM_Search();
    Update_RAM_Watch();
    YuiCaptureVideo();
 
@@ -664,45 +656,6 @@ int PERDXHandleEvents(void)
 
 //////////////////////////////////////////////////////////////////////////////
 
-int SkipKeyIsPressed=0;
-int DelayFactor = 5;
-
-int Check_Skip_Key()
-{
-	static time_t lastSkipTime = 0;
-	const int skipPressedNew = FrameAdvanceKeyDown;
-	static int checks = 0;
-
-	if(GetAsyncKeyState(VK_SPACE) & 0x8000)
-		FrameAdvanceKeyDown=1;
-	else
-		FrameAdvanceKeyDown=0;
-	
-	if(skipPressedNew && timeGetTime()-lastSkipTime >= 5)
-	{
-		checks++;
-		if(checks > 8000 + 60)
-			checks -= 8000;
-		lastSkipTime = timeGetTime();
-	}
-
-	if(skipPressedNew && (!SkipKeyIsPressed || ((checks > 60) && ((checks % DelayFactor) == 0))))
-	{
-		SkipKeyIsPressed=1;
-		return 1;
-	}
-	else {
-		if(!skipPressedNew && SkipKeyIsPressed)
-		{
-			SkipKeyIsPressed=0;
-			checks=0;
-		}
-		return 0;
-	}
-}
-
-//////////////////////////////////////////////////////////////////////////////
-
 BOOL CALLBACK EnumPeripheralsCallbackGamepad (LPCDIDEVICEINSTANCE lpddi, LPVOID pvRef)
 {
    if (GET_DIDEVICE_TYPE(lpddi->dwDevType) == DI8DEVTYPE_GAMEPAD ||
@@ -751,8 +704,8 @@ void PERDXListDevices(HWND control, int emulatetype)
 {
    LPDIRECTINPUT8 lpDI8temp = NULL;
 
-   if (DirectInput8Create(GetModuleHandle(NULL), DIRECTINPUT_VERSION,
-       &IID_IDirectInput8, (LPVOID *)&lpDI8temp, NULL) != DI_OK)
+   if (FAILED(DirectInput8Create(GetModuleHandle(NULL), DIRECTINPUT_VERSION,
+       &IID_IDirectInput8, (LPVOID *)&lpDI8temp, NULL)))
       return;
 
    numguids = 0;
@@ -912,12 +865,12 @@ int PERDXInitControlConfig(HWND hWnd, u8 padnum, int *controlmap, const char *in
          }
       }
 
-      if (DirectInput8Create(GetModuleHandle(NULL), DIRECTINPUT_VERSION,
-          &IID_IDirectInput8, (LPVOID *)&lpDI8temp, NULL) != DI_OK)
+      if (FAILED(DirectInput8Create(GetModuleHandle(NULL), DIRECTINPUT_VERSION,
+          &IID_IDirectInput8, (LPVOID *)&lpDI8temp, NULL)))
          return -1;
 
-      if (IDirectInput8_CreateDevice(lpDI8temp, &GUIDDevice[i], &lpDIDevicetemp,
-          NULL) != DI_OK)
+      if (FAILED(IDirectInput8_CreateDevice(lpDI8temp, &GUIDDevice[i], &lpDIDevicetemp,
+          NULL)))
       {
          IDirectInput8_Release(lpDI8temp);
          return -1;
@@ -925,7 +878,7 @@ int PERDXInitControlConfig(HWND hWnd, u8 padnum, int *controlmap, const char *in
 
       didc.dwSize = sizeof(DIDEVCAPS);
 
-      if (IDirectInputDevice8_GetCapabilities(lpDIDevicetemp, &didc) != DI_OK)
+      if (FAILED(IDirectInputDevice8_GetCapabilities(lpDIDevicetemp, &didc)))
       {
          IDirectInputDevice8_Release(lpDIDevicetemp);       
          IDirectInput8_Release(lpDI8temp);
@@ -987,12 +940,12 @@ int PERDXFetchNextPress(HWND hWnd, u32 guidnum, char *buttonname)
    DIDEVCAPS didc;
    int buttonid=-1;
 
-   if (DirectInput8Create(GetModuleHandle(NULL), DIRECTINPUT_VERSION,
-       &IID_IDirectInput8, (LPVOID *)&lpDI8temp, NULL) != DI_OK)
+   if (FAILED(DirectInput8Create(GetModuleHandle(NULL), DIRECTINPUT_VERSION,
+       &IID_IDirectInput8, (LPVOID *)&lpDI8temp, NULL)))
       return -1;
 
-   if (IDirectInput8_CreateDevice(lpDI8temp, &GUIDDevice[guidnum], &lpDIDevicetemp,
-       NULL) != DI_OK)
+   if (FAILED(IDirectInput8_CreateDevice(lpDI8temp, &GUIDDevice[guidnum], &lpDIDevicetemp,
+       NULL)))
    {
       IDirectInput8_Release(lpDI8temp);
       return -1;
@@ -1000,7 +953,7 @@ int PERDXFetchNextPress(HWND hWnd, u32 guidnum, char *buttonname)
 
    didc.dwSize = sizeof(DIDEVCAPS);
 
-   if (IDirectInputDevice8_GetCapabilities(lpDIDevicetemp, &didc) != DI_OK)
+   if (FAILED(IDirectInputDevice8_GetCapabilities(lpDIDevicetemp, &didc)))
    {
       IDirectInputDevice8_Release(lpDIDevicetemp);       
       IDirectInput8_Release(lpDI8temp);
@@ -1009,7 +962,7 @@ int PERDXFetchNextPress(HWND hWnd, u32 guidnum, char *buttonname)
 
    if (GET_DIDEVICE_TYPE(didc.dwDevType) == DI8DEVTYPE_KEYBOARD)
    {
-      if (IDirectInputDevice8_SetDataFormat(lpDIDevicetemp, &c_dfDIKeyboard) != DI_OK)
+      if (FAILED(IDirectInputDevice8_SetDataFormat(lpDIDevicetemp, &c_dfDIKeyboard)))
       {
          IDirectInputDevice8_Release(lpDIDevicetemp);       
          IDirectInput8_Release(lpDI8temp);
@@ -1019,7 +972,7 @@ int PERDXFetchNextPress(HWND hWnd, u32 guidnum, char *buttonname)
    else if (GET_DIDEVICE_TYPE(didc.dwDevType) == DI8DEVTYPE_GAMEPAD ||
            GET_DIDEVICE_TYPE(didc.dwDevType) == DI8DEVTYPE_JOYSTICK)
    {
-      if (IDirectInputDevice8_SetDataFormat(lpDIDevicetemp, &c_dfDIJoystick) != DI_OK)
+      if (FAILED(IDirectInputDevice8_SetDataFormat(lpDIDevicetemp, &c_dfDIJoystick)))
       {
          IDirectInputDevice8_Release(lpDIDevicetemp);       
          IDirectInput8_Release(lpDI8temp);
@@ -1028,7 +981,7 @@ int PERDXFetchNextPress(HWND hWnd, u32 guidnum, char *buttonname)
    }
    else if (GET_DIDEVICE_TYPE(didc.dwDevType) == DI8DEVTYPE_MOUSE)
    {
-      if (IDirectInputDevice8_SetDataFormat(lpDIDevicetemp, &c_dfDIMouse2) != DI_OK)
+      if (FAILED(IDirectInputDevice8_SetDataFormat(lpDIDevicetemp, &c_dfDIMouse2)))
       {
          IDirectInputDevice8_Release(lpDIDevicetemp);       
          IDirectInput8_Release(lpDI8temp);
@@ -1157,8 +1110,8 @@ LRESULT CALLBACK ButtonConfigDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam,
       {
          lpDIDevicetemp = (LPDIRECTINPUTDEVICE8)lParam;
 
-         if (IDirectInputDevice8_SetCooperativeLevel(lpDIDevicetemp, hDlg,
-              DISCL_FOREGROUND | DISCL_NONEXCLUSIVE | DISCL_NOWINKEY) != DI_OK)
+         if (FAILED(IDirectInputDevice8_SetCooperativeLevel(lpDIDevicetemp, hDlg,
+              DISCL_FOREGROUND | DISCL_NONEXCLUSIVE | DISCL_NOWINKEY)))
             return FALSE;
 
          dipdw.diph.dwSize = sizeof(DIPROPDWORD);
@@ -1168,7 +1121,7 @@ LRESULT CALLBACK ButtonConfigDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam,
          dipdw.dwData = 8; // should be enough
 
          // Setup Buffered input
-         if ((hr = IDirectInputDevice8_SetProperty(lpDIDevicetemp, DIPROP_BUFFERSIZE, &dipdw.diph)) != DI_OK)
+         if (FAILED((hr = IDirectInputDevice8_SetProperty(lpDIDevicetemp, DIPROP_BUFFERSIZE, &dipdw.diph))))
             return FALSE;
 
          if (!SetTimer(hDlg, 1, 100, NULL))
@@ -1214,8 +1167,8 @@ LRESULT CALLBACK ButtonConfigDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam,
             }
 
             // Poll events
-            if (IDirectInputDevice8_GetDeviceData(lpDIDevicetemp,
-                sizeof(DIDEVICEOBJECTDATA), didod, &size, 0) != DI_OK)
+            if (FAILED(IDirectInputDevice8_GetDeviceData(lpDIDevicetemp,
+                sizeof(DIDEVICEOBJECTDATA), didod, &size, 0)))
             {
                if (hr == DIERR_INPUTLOST || hr == DIERR_NOTACQUIRED)
                {
@@ -1227,7 +1180,7 @@ LRESULT CALLBACK ButtonConfigDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam,
 
             didc.dwSize = sizeof(DIDEVCAPS);
 
-            if (IDirectInputDevice8_GetCapabilities(lpDIDevicetemp, &didc) != DI_OK)
+            if (FAILED(IDirectInputDevice8_GetCapabilities(lpDIDevicetemp, &didc)))
                return TRUE;
 
             if (GET_DIDEVICE_TYPE(didc.dwDevType) == DI8DEVTYPE_KEYBOARD)
