@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with Yabause; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
 #ifndef PERDX_H
@@ -23,6 +23,7 @@
 #define DIRECTINPUT_VERSION 0x0800
 #include <dinput.h>
 #include "dx.h"
+#include "../peripheral.h"
 
 #define PERCORE_DIRECTX 2
 
@@ -33,6 +34,7 @@ extern u32 numguids;
 extern const char * pad_names[];
 extern PerPad_struct *pad[12];
 extern u32 numpads;
+extern int porttype[2];
 
 typedef struct
 {
@@ -53,5 +55,5 @@ void PERDXLoadDevices(char *inifilename);
 void PERDXListDevices(HWND control, int emulatetype);
 int PERDXInitControlConfig(HWND hWnd, u8 padnum, int *controlmap, const char *inifilename);
 int PERDXFetchNextPress(HWND hWnd, u32 guidnum, char *buttonname);
-BOOL PERDXWriteGUID(u32 guidnum, u8 padnum, LPCTSTR inifilename);
+BOOL PERDXWriteGUID(u32 guidnum, u8 padnum, LPCSTR inifilename);
 #endif
