@@ -24,6 +24,7 @@
 #include "../../cs0.h"
 #include "../../cs2.h"
 #include "../resource.h"
+#include "../../yabause.h"
 #include "settings.h"
 
 char biosfilename[MAX_PATH] = "\0";
@@ -41,6 +42,8 @@ int carttype;
 int num_cdroms=0;
 char drive_list[24];
 
+void StartGame();
+
 //////////////////////////////////////////////////////////////////////////////
 
 void GenerateCDROMList(HWND hWnd)
@@ -691,6 +694,8 @@ LRESULT CALLBACK BasicSettingsDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam,
 
                if (cdromchanged && nocorechange == 0)
                {
+					StartGame();
+
 #ifndef USETHREADS
                   if (IsPathCdrom(cdrompath))
                      Cs2ChangeCDCore(CDCORE_SPTI, cdrompath);
@@ -701,6 +706,7 @@ LRESULT CALLBACK BasicSettingsDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam,
                   changecore |= 1;
                   while (corechanged == 0) { Sleep(0); }
 #endif
+				  YabauseReset();
                }
 
           		SetWindowLong(hDlg,	DWL_MSGRESULT, PSNRET_NOERROR);
