@@ -42,7 +42,7 @@
 #include "bios.h"
 #include "movie.h"
 #ifdef HAVE_LIBSDL
- #ifdef __APPLE__
+ #if defined(__APPLE__) || defined(GEKKO)
   #include <SDL/SDL.h>
  #else
   #include "SDL.h"
@@ -58,7 +58,6 @@
 #endif
 #ifdef GEKKO
 #include <ogc/lwp_watchdog.h>
-extern long long gettime();
 #endif
 #ifdef PSP
 #include "psp/common.h"
@@ -101,39 +100,31 @@ void print_usage(const char *program_name) {
 
 void YabauseChangeTiming(int freqtype) {
    // Setup all the variables related to timing
+
+   const double freq_base = yabsys.IsPal ? 28437500.0
+      : (39375000.0 / 11.0) * 8.0;  // i.e. 8 * 3.579545... = 28.636363... MHz
+   const double freq_mult = (freqtype == CLKTYPE_26MHZ) ? 15.0/16.0 : 1.0;
+   const double freq_shifted = (freq_base * freq_mult) * (1 << YABSYS_TIMING_BITS);
+   const double usec_shifted = 1.0e6 * (1 << YABSYS_TIMING_BITS);
+   const double deciline_time = yabsys.IsPal ? 1.0 /  50        / 313 / 10
+                                             : 1.0 / (60/1.001) / 263 / 10;
+
    yabsys.DecilineCount = 0;
    yabsys.LineCount = 0;
    yabsys.CurSH2FreqType = freqtype;
-
-   switch (freqtype)
-   {
-      case CLKTYPE_26MHZ:
-         if (!yabsys.IsPal)
-            yabsys.DecilineStop = 26846587 / 263 / 10 / 60; // I'm guessing this is wrong
-         else
-            yabsys.DecilineStop = 26846587 / 312 / 10 / 50; // I'm guessing this is wrong
-
-         yabsys.Duf = 26846587 / 100000;
-
-         break;
-      case CLKTYPE_28MHZ:
-         if (!yabsys.IsPal)
-            yabsys.DecilineStop = 28636360 / 263 / 10 / 60; // I'm guessing this is wrong
-         else
-            yabsys.DecilineStop = 28636360 / 312 / 10 / 50; // I'm guessing this is wrong
-
-         yabsys.Duf = 28636360 / 100000;
-         break;
-      default: break;
-   }
-
-   yabsys.CycleCountII = 0;
+   yabsys.DecilineStop = (u32) (freq_shifted * deciline_time + 0.5);
+   yabsys.SH2CycleFrac = 0;
+   yabsys.DecilineUsec = (u32) (usec_shifted * deciline_time + 0.5);
+   yabsys.UsecFrac = 0;
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
 int YabauseInit(yabauseinit_struct *init)
-{     
+{
+   // Need to set this first, so init routines see it
+   yabsys.UseThreads = init->usethreads;
+
    // Initialize both cpu's
    if (SH2Init(init->sh2coretype) != 0)
    {
@@ -156,8 +147,18 @@ int YabauseInit(yabauseinit_struct *init)
    if (LoadBackupRam(init->buppath) != 0)
       FormatBackupRam(BupRam, 0x10000);
 
+   BupRamWritten = 0;
+
    bupfilename = init->buppath;
 
+   if (CartInit(init->cartpath, init->carttype) != 0)
+   {
+      YabSetError(YAB_ERR_CANNOTINIT, _("Cartridge"));
+      return -1;
+   }
+
+   MappedMemoryInit();
+
    if (VideoInit(init->vidcoretype) != 0)
    {
       YabSetError(YAB_ERR_CANNOTINIT, _("Video"));
@@ -171,12 +172,6 @@ int YabauseInit(yabauseinit_struct *init)
       return -1;
    }
 
-   if (CartInit(init->cartpath, init->carttype) != 0)
-   {
-      YabSetError(YAB_ERR_CANNOTINIT, _("Cartridge"));
-      return -1;
-   }
-
    if (Cs2Init(init->carttype, init->cdcoretype, init->cdpath, init->mpegpath, init->netlinksetting) != 0)
    {
       YabSetError(YAB_ERR_CANNOTINIT, _("CS2"));
@@ -213,7 +208,7 @@ int YabauseInit(yabauseinit_struct *init)
       return -1;
    }
 
-   if (SmpcInit(init->regionid) != 0)
+   if (SmpcInit(init->regionid, init->clocksync, init->basetime) != 0)
    {
       YabSetError(YAB_ERR_CANNOTINIT, _("SMPC"));
       return -1;
@@ -225,13 +220,12 @@ int YabauseInit(yabauseinit_struct *init)
       return -1;
    }
 
-   MappedMemoryInit();
-
-   YabauseSetVideoFormat(init->flags & 0x1);
+   YabauseSetVideoFormat(init->videoformattype);
    YabauseChangeTiming(CLKTYPE_26MHZ);
-   
+   yabsys.DecilineMode = 1;
+
    if (init->frameskip)
-       EnableAutoFrameSkip();
+      EnableAutoFrameSkip();
 
    if (init->biospath != NULL && strlen(init->biospath))
    {
@@ -247,6 +241,12 @@ int YabauseInit(yabauseinit_struct *init)
 
    yabsys.usequickload = 0;
 
+   #if defined(SH2_DYNAREC)
+   if(SH2Core->id==2) {
+     sh2_dynarec_init();
+   }
+   #endif
+
    YabauseResetNoLoad();
 
    if (yabsys.usequickload || yabsys.emulatebios)
@@ -269,6 +269,11 @@ int YabauseInit(yabauseinit_struct *init)
 //////////////////////////////////////////////////////////////////////////////
 
 void YabauseDeInit(void) {
+   #if defined(SH2_DYNAREC)
+   if(SH2Core->id==2) {
+     sh2_dynarec_cleanup();
+   }
+   #endif
    SH2DeInit();
 
    if (BiosRom)
@@ -306,6 +311,12 @@ void YabauseDeInit(void) {
 
 //////////////////////////////////////////////////////////////////////////////
 
+void YabauseSetDecilineMode(int on) {
+   yabsys.DecilineMode = (on != 0);
+}
+
+//////////////////////////////////////////////////////////////////////////////
+
 void YabauseResetNoLoad(void) {
    SH2Reset(MSH2);
    YabauseStopSlave();
@@ -317,7 +328,6 @@ void YabauseResetNoLoad(void) {
    Cs2Reset();
    ScuReset();
    ScspReset();
-   yabsys.IsM68KRunning = 0;
    Vdp1Reset();
    Vdp2Reset();
    SmpcReset();
@@ -383,83 +393,146 @@ int YabauseExec(void) {
 }
 
 //////////////////////////////////////////////////////////////////////////////
+#ifndef USE_SCSP2
+int saved_centicycles;
+#endif
 
 int YabauseEmulate(void) {
-   int M68k_cycles;              // Integral M68k cycles to execute this time
-   static int M68k_centicycles;  // Fractional cycle counter for M68k
+   int oneframeexec = 0;
 
-   int oneframeexec=0;
+   const u32 cyclesinc =
+      yabsys.DecilineMode ? yabsys.DecilineStop : yabsys.DecilineStop * 10;
+   const u32 usecinc =
+      yabsys.DecilineMode ? yabsys.DecilineUsec : yabsys.DecilineUsec * 10;
+#ifndef USE_SCSP2
+   unsigned int m68kcycles;       // Integral M68k cycles per call
+   unsigned int m68kcenticycles;  // 1/100 M68k cycles per call
+   
+   if (yabsys.IsPal)
+   {
+      /* 11.2896MHz / 50Hz / 313 lines / 10 calls/line = 72.20 cycles/call */
+      m68kcycles = yabsys.DecilineMode ? 72 : 722;
+      m68kcenticycles = yabsys.DecilineMode ? 20 : 0;
+   }
+   else
+   {
+      /* 11.2896MHz / 60Hz / 263 lines / 10 calls/line = 71.62 cycles/call */
+      m68kcycles = yabsys.DecilineMode ? 71 : 716;
+      m68kcenticycles = yabsys.DecilineMode ? 62 : 20;
+   }
+#endif
 
    DoMovie();
 
+   #if defined(SH2_DYNAREC)
+   if(SH2Core->id==2) {
+     if (yabsys.IsPal)
+       YabauseDynarecOneFrameExec(722,0); // m68kcycles,m68kcenticycles
+     else
+       YabauseDynarecOneFrameExec(716,20);
+     return 0;
+   }
+   #endif
+
    while (!oneframeexec)
    {
-#ifdef NO_DECILINE
-      const unsigned int sh2cycles = yabsys.DecilineStop * 10;
-#else
-      const unsigned int sh2cycles = yabsys.DecilineStop;
-#endif
-
       PROFILE_START("Total Emulation");
 
-#ifdef NO_DECILINE
+      if (yabsys.DecilineMode) {
 
-      PROFILE_START("MSH2");
-      SH2Exec(MSH2, sh2cycles - yabsys.DecilineStop);
-      PROFILE_STOP("MSH2");
-      PROFILE_START("SSH2");
-      if (yabsys.IsSSH2Running)
-         SH2Exec(SSH2, sh2cycles - yabsys.DecilineStop);
-      PROFILE_STOP("SSH2");
+         // Since we run the SCU with half the number of cycles we send
+         // to SH2Exec(), we always compute an even number of cycles here
+         // and leave any odd remainder in SH2CycleFrac.
+         u32 sh2cycles;
+         yabsys.SH2CycleFrac += cyclesinc;
+         sh2cycles = (yabsys.SH2CycleFrac >> (YABSYS_TIMING_BITS + 1)) << 1;
+         yabsys.SH2CycleFrac &= ((YABSYS_TIMING_MASK << 1) | 1);
 
-      PROFILE_START("hblankin");
-      Vdp2HBlankIN();
-      PROFILE_STOP("hblankin");
+         PROFILE_START("MSH2");
+         SH2Exec(MSH2, sh2cycles);
+         PROFILE_STOP("MSH2");
 
-      PROFILE_START("MSH2");
-      SH2Exec(MSH2, yabsys.DecilineStop);
-      PROFILE_STOP("MSH2");
-      PROFILE_START("SSH2");
-      if (yabsys.IsSSH2Running)
-         SH2Exec(SSH2, yabsys.DecilineStop);
-      PROFILE_STOP("SSH2");
-
-#else // !NO_DECILINE
-
-      PROFILE_START("MSH2");
-      SH2Exec(MSH2, sh2cycles);
-      PROFILE_STOP("MSH2");
-
-      PROFILE_START("SSH2");
-      if (yabsys.IsSSH2Running)
-         SH2Exec(SSH2, sh2cycles);
-      PROFILE_STOP("SSH2");
+         PROFILE_START("SSH2");
+         if (yabsys.IsSSH2Running)
+            SH2Exec(SSH2, sh2cycles);
+         PROFILE_STOP("SSH2");
 
+#ifdef USE_SCSP2
+         PROFILE_START("SCSP");
+         ScspExec(1);
+         PROFILE_STOP("SCSP");
 #endif
 
-      PROFILE_START("SCU");
-      ScuExec(sh2cycles / 2);
-      PROFILE_STOP("SCU");
+         yabsys.DecilineCount++;
+         if(yabsys.DecilineCount == 9)
+         {
+            // HBlankIN
+            PROFILE_START("hblankin");
+            Vdp2HBlankIN();
+            PROFILE_STOP("hblankin");
+         }
+
+         PROFILE_START("SCU");
+         ScuExec(sh2cycles / 2);
+         PROFILE_STOP("SCU");
+
+      } else {  // !DecilineMode
+
+         const u32 decilinecycles = yabsys.DecilineStop >> YABSYS_TIMING_BITS;
+         u32 sh2cycles;
+         yabsys.SH2CycleFrac += cyclesinc;
+         sh2cycles = (yabsys.SH2CycleFrac >> (YABSYS_TIMING_BITS + 1)) << 1;
+         yabsys.SH2CycleFrac &= ((YABSYS_TIMING_MASK << 1) | 1);
+
+         PROFILE_START("MSH2");
+         SH2Exec(MSH2, sh2cycles - decilinecycles);
+         PROFILE_STOP("MSH2");
+         PROFILE_START("SSH2");
+         if (yabsys.IsSSH2Running)
+            SH2Exec(SSH2, sh2cycles - decilinecycles);
+         PROFILE_STOP("SSH2");
 
-#ifndef NO_DECILINE
-      yabsys.DecilineCount++;
-      if(yabsys.DecilineCount == 9)
-      {
-         // HBlankIN
          PROFILE_START("hblankin");
          Vdp2HBlankIN();
          PROFILE_STOP("hblankin");
-      }
-      else if (yabsys.DecilineCount == 10)
+
+         PROFILE_START("MSH2");
+         SH2Exec(MSH2, decilinecycles);
+         PROFILE_STOP("MSH2");
+         PROFILE_START("SSH2");
+         if (yabsys.IsSSH2Running)
+            SH2Exec(SSH2, decilinecycles);
+         PROFILE_STOP("SSH2");
+
+#ifdef USE_SCSP2
+         PROFILE_START("SCSP");
+         ScspExec(10);
+         PROFILE_STOP("SCSP");
 #endif
+
+         PROFILE_START("SCU");
+         ScuExec(sh2cycles / 2);
+         PROFILE_STOP("SCU");
+
+      }  // if (yabsys.DecilineMode)
+
+#ifndef USE_SCSP2
+      PROFILE_START("68K");
+      M68KSync();  // Wait for the previous iteration to finish
+      PROFILE_STOP("68K");
+#endif
+
+      if (!yabsys.DecilineMode || yabsys.DecilineCount == 10)
       {
          // HBlankOUT
          PROFILE_START("hblankout");
          Vdp2HBlankOUT();
          PROFILE_STOP("hblankout");
+#ifndef USE_SCSP2
          PROFILE_START("SCSP");
          ScspExec();
          PROFILE_STOP("SCSP");
+#endif
          yabsys.DecilineCount = 0;
          yabsys.LineCount++;
          if (yabsys.LineCount == yabsys.VBlankLineCount)
@@ -482,71 +555,38 @@ int YabauseEmulate(void) {
          }
       }
 
-#ifdef PSP_TIMING_TWEAKS
-      yabsys.CycleCountII += sh2cycles;
-#else
-      yabsys.CycleCountII += sh2cycles + MSH2->cycles;
-#endif
+      yabsys.UsecFrac += usecinc;
+      PROFILE_START("SMPC");
+      SmpcExec(yabsys.UsecFrac >> YABSYS_TIMING_BITS);
+      PROFILE_STOP("SMPC");
+      PROFILE_START("CDB");
+      Cs2Exec(yabsys.UsecFrac >> YABSYS_TIMING_BITS);
+      PROFILE_STOP("CDB");
+      yabsys.UsecFrac &= YABSYS_TIMING_MASK;
 
+#ifndef USE_SCSP2
       {
-         u32 usec = 0;
-         while (yabsys.CycleCountII > yabsys.Duf)
-         {
-             yabsys.CycleCountII -= yabsys.Duf;
-             usec += 10;
-         }
-         if (usec > 0)
-         {
-            PROFILE_START("SMPC");
-            SmpcExec(usec);
-            PROFILE_STOP("SMPC");
-            PROFILE_START("CDB");
-            Cs2Exec(usec);
-            PROFILE_STOP("CDB");
-         }
-      }
+         int cycles;
 
-      PROFILE_START("68K");
-#ifdef NO_DECILINE
-      if (yabsys.IsPal)
-      {
-         /* 11.2896MHz / 50Hz / 262.5 lines / 10 calls/line = 86.01 cycles/call */
-         M68k_cycles = 860;
-         M68k_centicycles += 10;
-      }
-      else
-      {
-         /* 11.2896MHz / 60Hz / 262.5 lines / 10 calls/line = 71.68 cycles/call */
-         M68k_cycles = 716;
-         M68k_centicycles += 80;
-      }
-      if (M68k_centicycles >= 100) {
-         M68k_cycles++;
-         M68k_centicycles -= 100;
-      }
-#else
-      if (yabsys.IsPal)
-      {
-         /* 11.2896MHz / 50Hz / 262.5 lines / 10 calls/line = 86.01 cycles/call */
-         M68k_cycles = 86;
-         M68k_centicycles += 1;
-      }
-      else
-      {
-         /* 11.2896MHz / 60Hz / 262.5 lines / 10 calls/line = 71.68 cycles/call */
-         M68k_cycles = 71;
-         M68k_centicycles += 68;
-      }
-      if (M68k_centicycles >= 100) {
-         M68k_cycles++;
-         M68k_centicycles -= 100;
+         PROFILE_START("68K");
+         cycles = m68kcycles;
+	 saved_centicycles += m68kcenticycles;
+         if (saved_centicycles >= 100) {
+            cycles++;
+            saved_centicycles -= 100;
+         }
+         M68KExec(cycles);
+         PROFILE_STOP("68K");
       }
 #endif
-      M68KExec(M68k_cycles);
-      PROFILE_STOP("68K");
 
       PROFILE_STOP("Total Emulation");
    }
+
+#ifndef USE_SCSP2
+   M68KSync();
+#endif
+
    return 0;
 }
 
@@ -555,9 +595,11 @@ int YabauseEmulate(void) {
 void YabauseStartSlave(void) {
    if (yabsys.emulatebios)
    {
+      SH2GetRegisters(SSH2, &SSH2->regs);
       SSH2->regs.R[15] = 0x06001000;
       SSH2->regs.VBR = 0x06000400;
       SSH2->regs.PC = MappedMemoryReadLong(0x06000250);
+      SH2SetRegisters(SSH2, &SSH2->regs);
    }
    else
       SH2PowerOn(SSH2);
@@ -612,7 +654,8 @@ void YabauseSetVideoFormat(int type) {
 #elif defined(HAVE_LIBSDL)
    yabsys.tickfreq = 1000;
 #endif
-   yabsys.OneFrameTime = yabsys.tickfreq / (type ? 50 : 60);
+   yabsys.OneFrameTime =
+      type ? (yabsys.tickfreq / 50) : (yabsys.tickfreq * 1001 / 60000);
    Vdp2Regs->TVSTAT = Vdp2Regs->TVSTAT | (type & 0x1);
    ScspChangeVideoFormat(type);
    YabauseChangeTiming(yabsys.CurSH2FreqType);
@@ -687,6 +730,7 @@ void YabauseSpeedySetup(void)
    Cs2Area->satauth = 4;
 
    // Set Master SH2 registers accordingly
+   SH2GetRegisters(MSH2, &MSH2->regs);
    for (i = 0; i < 15; i++)
       MSH2->regs.R[i] = 0x00000000;
    MSH2->regs.R[15] = 0x06002000;
@@ -696,6 +740,7 @@ void YabauseSpeedySetup(void)
    MSH2->regs.MACH = 0x00000000;
    MSH2->regs.MACL = 0x00000000;
    MSH2->regs.PR = 0x00000000;
+   SH2SetRegisters(MSH2, &MSH2->regs);
 
    // Set SCU registers to sane states
    ScuRegs->D1AD = ScuRegs->D2AD = 0;
@@ -828,6 +873,8 @@ int YabauseQuickLoadGame(void)
          lgpartition->numblocks = 0;
       }
 
+      SH2WriteNotify(0x6002000, blocks<<11);
+
       // Ok, now that we've loaded the ip, now it's time to load the
       // First Program
 
@@ -898,8 +945,12 @@ int YabauseQuickLoadGame(void)
          lgpartition->numblocks = 0;
       }
 
+      SH2WriteNotify(addr, blocks<<11);
+
       // Now setup SH2 registers to start executing at ip code
+      SH2GetRegisters(MSH2, &MSH2->regs);
       MSH2->regs.PC = 0x06002E00;
+      SH2SetRegisters(MSH2, &MSH2->regs);
    }
    else
    {
@@ -917,4 +968,4 @@ int YabauseQuickLoadGame(void)
    return 0;
 }
 
-//////////////////////////////////////////////////////////////////////////////
\ No newline at end of file
+//////////////////////////////////////////////////////////////////////////////
