@@ -65,8 +65,14 @@ typedef struct {
 	unsigned int h;
 	int flip;
 	int priority;
+	int dst;
 } YglSprite;
 
+typedef struct {
+	float x;
+	float y;
+} YglCache;
+
 typedef struct {
 	unsigned int * textdata;
 	unsigned int w;
@@ -96,7 +102,7 @@ void YglTMAllocate(YglTexture *, unsigned int, unsigned int, unsigned int *, uns
 
 typedef struct {
 	int * quads;
-	int * textcoords;
+	float * textcoords;
 	int currentQuad;
 	int maxQuad;
 #ifdef USEMICSHADERS
@@ -123,11 +129,11 @@ int YglGLInit(int, int);
 int YglScreenInit(int r, int g, int b, int d);
 int YglInit(int, int, unsigned int);
 void YglDeInit(void);
-int * YglQuad(YglSprite *, YglTexture *);
-void YglCachedQuad(YglSprite *, int *);
+float * YglQuad(YglSprite *, YglTexture *,YglCache * c);
+void YglCachedQuad(YglSprite *, YglCache *);
 #ifdef USEMICSHADERS
-int * YglQuad2(YglSprite *, YglTexture *, YglColor *);
-void YglCachedQuad2(YglSprite *, int *, YglColor *);
+float * YglQuad2(YglSprite *, YglTexture *, YglColor *,YglCache * c);
+void YglCachedQuad2(YglSprite *, YglCache *, YglColor *);
 #endif
 void YglRender(void);
 void YglReset(void);
@@ -135,10 +141,31 @@ void YglShowTexture(void);
 void YglChangeResolution(int, int);
 void YglOnScreenDebugMessage(char *, ...);
 
-
-int * YglIsCached(u32);
-void YglCache(u32, int *);
+int YglIsCached(u32,YglCache *);
+void YglCacheAdd(u32,YglCache *);
 void YglCacheReset(void);
 
+#ifdef USEMICSHADERS
+
+#if 0  // Does anything need this?  It breaks a bunch of prototypes if
+       // GLchar is typedef'd instead of #define'd  --AC
+#ifndef GLchar
+#define GLchar GLbyte
+#endif
+#endif  // 0
+
+extern GLuint (STDCALL *pfglCreateProgram)(void);
+extern GLuint (STDCALL *pfglCreateShader)(GLenum);
+extern void (STDCALL *pfglShaderSource)(GLuint,GLsizei,const GLchar **,const GLint *);
+extern void (STDCALL *pfglCompileShader)(GLuint);
+extern void (STDCALL *pfglAttachShader)(GLuint,GLuint);
+extern void (STDCALL *pfglLinkProgram)(GLuint);
+extern void (STDCALL *pfglUseProgram)(GLuint);
+extern GLint (STDCALL *pfglGetUniformLocation)(GLuint,const GLchar *);
+extern void (STDCALL *pfglUniform1i)(GLint,GLint);
+extern void (STDCALL *pfglGetShaderInfoLog)(GLuint,GLsizei,GLsizei *,GLchar *);
+
+#endif
+
 #endif
 #endif
